/*

-Procedure  checkck ( Check for S/C and ASPERA CK Data Availability )
 
-Abstract
 
   Returns flags indicating if MEX s/c and ASPERA scanner orientation
   data is available for specified ET in loaded CK files.
 
-Required_Reading
 
   CK Required Reading (ck.req)
   SCLK Required Reading (sclk.req)
   Latest MEX FK File (mex_vXX.tf)
 
*/

   #include "SpiceUsr.h"
 
   void checkck( SpiceDouble         et,
                 SpiceBoolean      * gotscatt, 
                 SpiceBoolean      * gotscanatt )

/*
 
-Brief_I/O
 
   Variable  I/O  Description
   --------  ---  --------------------------------------------
   et         I   Ephemeris time of interest
   gotscatt   O   1 if s/c orientation is available; 0 otherwise
   gotscanatt O   1 if scanner orientation is available; 0 otherwise

-Detailed Input
 
   N/A.
   
-Detailed Output

   N/A.

-Exceptions
 
   1) If SCLK data have not been loaded prior to calling this function,
      then an appropriate error will be signaled by the SCLK subsystem.
 
-Files
 
   MEX SCLK file must be loaded prior to calling this function. MEX s/c
   CK and ASPERA scanner CK should be loaded prior to calling this
   function.
  
-Particulars

   MEX s/c orientation in the loaded CK files is assumed to be provided 
   for the NAIF ID -41001 (attached to the MEX_SC_REF frame.)
 
   ASPERA scanner orientation in the loaded CK files is assumed to be 
   provided for the NAIF ID -41111 (attached to the MEX_ASPERA_SCANNER 
   frame.)
 
-Examples
 
   In this example the function is called to check for orientation some
   time in the middle of the Earth-Mars cruise. A CK providing s/c
   orientation is loaded. No CK providing orientation for the ASPERA
   scanner is loaded.

      #include <stdio.h>
      #include "SpiceUsr.h"

      void checkck( SpiceDouble         et,
                    SpiceBoolean      * gotscatt, 
                    SpiceBoolean      * gotscanatt );

      int main( int argc, char * argv[] )
      {
         SpiceDouble             et;
         SpiceBoolean            gotscatt;
         SpiceBoolean            gotscanatt;

         /.
           Load LSK, MEX SCLK, and MEX s/c orientation CK files.
         ./
         furnsh_c( "naif0007.tls" );
         furnsh_c( "mex_030618_step.tsc" );
         furnsh_c( "ATNM_PSTPIX_DBP030602191822_00021.bc" );

         /.
           Convert UTC to ET.
         ./
         str2et_c ( "2003-09-09 12:00", &et );

         /.
           Check for CK data.
         ./
         checkck ( et, &gotscatt, &gotscanatt );

         /.
           Report data availability.
         ./
         printf ( "SPACECRAFT ATTITUDE: %d\n", gotscatt );
         printf ( "SCANNER ATTITUDE:    %d\n", gotscanatt );

         return(0);

      }

-Restrictions
 
   N/A.
 
-Literature_References
 
   None.
 
-Author_and_Institution
 
   B.V. Semenov    (JPL)
 
-Version
 
   -checkck Version 1.0.0, 2003-JUL-18 (BVS)

      Initial version
 
-Index_Entries
 
   None.

-& 
*/
 
{  /* Begin "check CK data" */

   SpiceDouble             sclkdp;
   SpiceDouble             cmat   [3][3];
   SpiceDouble             clkout;

   /*
     To verify that loaded CK files (if any) contain spacecraft and
     scanner orientation data one needs to call a lower level CK access
     routine, ckgp_c, which requires encoded SCLK as input time, so ET
     should be converted to it first.
   */
   sce2t_c  ( -41, et, &sclkdp );

   /*
     MEX s/c orientation is attached to NAIF ID -41001 (MEX_SC_REF)
     frame. Call ckgp_c to see if there is orientation available for
     this ID at specified time. The only ckgp_c output of interest is
     the last one -- the flag indicating that the orientation is
     available.
   */
   ckgp_c  ( -41001, sclkdp, 0.0, "J2000", cmat, &clkout, gotscatt );

   /*
     ASPERA scanner orientation is attached to NAIF ID -41111
     (MEX_ASPERA_SCANNER) frame. Again call ckgp_c to see if there is
     orientation available for this ID at specified time. As before,
     the only ckgp_c output of interest is the last one -- the flag
     indicating that the orientation is available.
   */
   ckgp_c  ( -41111, sclkdp, 0.0, "J2000", cmat, &clkout, gotscanatt );
   

} /* End of "check CK data" */
