/*

-Procedure  getik ( Get ASPERA IK Data )
 
-Abstract
 
   Return IK data for a given ASPERA sensor.
 
-Required_Reading
 
   KERNEL Required Reading (kernel.req)
   Latest ASPERA IK File (mex_aspera_vXX.ti)
   Latest MEX FK File (mex_vXX.tf)
 
*/

   #include "SpiceUsr.h"
   #include <string.h>
 
   void getik( ConstSpiceChar    * sensor,
               SpiceChar         * sensnm,
               SpiceInt          * sensid,
               SpiceInt            secnum[2],
               SpiceDouble         secsiz[2],
               SpiceChar         * secfrm,
               SpiceDouble       * secdir )

/*
 
-Brief_I/O
 
   Variable  I/O  Description
   --------  ---  --------------------------------------------
   sensor     I   ASPERA sensor acronym -- ELS, NPD, NPI, or IMA
   sensnm     O   Full NAIF name of the sensor
   sensid     O   NAIF ID of the sensor
   secnum     O   Sector grid dimensions (in-plane, normal to plane)    
   secsiz     O   Sector size, degrees (in-plane, normal to plane) 
   secfrm     O   Frame in this in-plane view directions are defined
   secdir     O   In-plane sector center view directions
 
-Detailed Input
 
   N/A.
   
-Detailed Output

   N/A.

-Exceptions
 
   1) If NAIF name <-> ID mapping for MEX instruments provided in the
      MEX FK file have not been loaded, then the error
      'NOIDMAPPINGDATA' is signaled.
 
   2) If any of the expected IK keywords provided in the ASPERA IK file
      have not been found in the POOL, then the error 'NOIKDATA' is
      signaled.

   3) If not enough space should be provided for the sensnm, secfrm,
      secdir, then memory violations will occur.
 
-Files
 
   MEX FK and ASPERA IK files must be loaded prior to calling this
   function.
  
-Particulars

   MEX FK file loaded prior to calling this routine should contain 
   MEX instrument naif <-> ID mappings.

   ASPERA IK file loaded prior to calling this routine should contain 
   the following keywords for each of the sensors:

      INS#_NUMBER_OF_SECTORS
      INS#_SECTOR_SIZE
      INS#_FRAME
      INS#_SECTOR_DIRECTIONS
 
-Examples
 
   In this example the function gets and prints out IK data for ELS.

      #include <stdio.h>
      #include "SpiceUsr.h"

      void getik( ConstSpiceChar    * sensor,
                  SpiceChar         * sensnm,
                  SpiceInt          * sensid,
                  SpiceInt            secnum[2],
                  SpiceDouble         secsiz[2],
                  SpiceChar         * secfrm,
                  SpiceDouble       * secdir );

      int main( int argc, char * argv[] )
      {
         SpiceChar               sensnm [33];
         SpiceInt                sensid;
         SpiceInt                secnum [2];
         SpiceDouble             secsiz [2];
         SpiceChar               secfrm [33];
         SpiceDouble             secdir [32][3];        
         SpiceInt                i;

         /.
           Load MEX FK and ASPERA IK files.
         ./
         furnsh_c( "mex_v03.tf" );
         furnsh_c( "mex_aspera_v01.ti" );

         /.
           Get ELS IK data.
         ./
         getik( "ELS", sensnm, &sensid, secnum, secsiz, secfrm, 
                ( SpiceDouble * ) secdir );

         /.
           Report ELS IK data.
         ./
         printf( "SENSOR NAIF NAME:  %s           \n", sensnm               );
         printf( "SENSOR NAIF ID:    %d           \n", sensid               );
         printf( "NUMBER OF SECTORS: %d %d        \n", secnum[0], secnum[1] );
         printf( "SECTOR SIZE:       %12.3f %12.3f\n", secsiz[0], secsiz[1] );
         printf( "REFERENCE FRAME:   %s           \n", secfrm               );
         printf( "SECTOR VIEW DIRECTIONS:         \n"                       );
         for ( i = 0; i < secnum[0]; i++ )
         {
            printf( "   %12.9f %12.9f %12.9f\n", 
                    secdir[i][0], secdir[i][1], secdir[i][2] );
         }
         
         return(0);

      }

-Restrictions
 
   N/A
 
-Literature_References
 
   None.
 
-Author_and_Institution
 
   B.V. Semenov    (JPL)
 
-Version
 
   -getik Version 1.0.0, 2003-JUL-18 (BVS)

      Initial version
 
-Index_Entries
 
   None.

-& 
*/
 
{  /* Begin "get IK data */

   SpiceChar               ikkwd  [33];
   SpiceInt                n;

   SpiceBoolean            found;

   /*
     Since all IK variable names contain NAIF ID of the instrument, the
     inputs sensor acronym, NNN, needs to be expanded into its full
     name, MEX_ASPERA_NNN, which then can be uses to find the sensor's
     NAIF ID code.
   */
   strcpy   ( sensnm, "MEX_ASPERA_" );
   strcat   ( sensnm, sensor );
   bodn2c_c ( sensnm, sensid, &found );
   if ( ! found  )
   {
     setmsg_c ( "Cannot determine NAIF ID for #." );
     errch_c  ( "#", sensnm );
     sigerr_c ( "NOIDMAPPINGDATA" );
   }

   /*
     Now IK routines can be used to retrieve loaded data. First,
     retrieve the number of sectors provided in the
     INS-NNNNNN_NUMBER_OF_SECTORS keyword (here -NNNNNN is the NAIF ID
     of the sensor.)
   */
   strcpy   ( ikkwd, "INS#_NUMBER_OF_SECTORS" );
   repmi_c  ( ikkwd, "#", *sensid, 33, ikkwd );
   gipool_c ( ikkwd, 0, 2,            &n, secnum, &found );
   if ( ! found ) {
     setmsg_c ( "Loaded IK does not contain #." );
     errch_c  ( "#", ikkwd );
     sigerr_c ( "NOIKDATA" );
   }

   /*
     Second, retrieve the sector size provided in the
     INS-NNNNNN_SECTOR_SIZE keyword.
   */   
   strcpy   ( ikkwd, "INS#_SECTOR_SIZE" );
   repmi_c  ( ikkwd, "#", *sensid, 33, ikkwd );
   gdpool_c ( ikkwd, 0, 2,            &n, secsiz, &found );
   if ( ! found ) {
     setmsg_c ( "Loaded IK does not contain #." );
     errch_c  ( "#", ikkwd );
     sigerr_c ( "NOIKDATA" );
   }

   /*
     Third, retrieve the frame in which sector view direction are
     defined. It is provided in the INS-NNNNNN_FRAME keyword.
   */   
   strcpy   ( ikkwd, "INS#_FRAME" );
   repmi_c  ( ikkwd, "#", *sensid, 33, ikkwd );
   gcpool_c ( ikkwd, 0, 1, 33,    &n, secfrm, &found );
   if ( ! found ) {
     setmsg_c ( "Loaded IK does not contain #." );
     errch_c  ( "#", ikkwd );
     sigerr_c ( "NOIKDATA" );
   }

   /*
     Last, retrieve the sector view directions provided in the 
     INS-NNNNNN_SECTOR_DIRECTIONS keyword.
   */   
   strcpy   ( ikkwd, "INS#_SECTOR_DIRECTIONS" );
   repmi_c  ( ikkwd, "#", *sensid, 33, ikkwd );
   gdpool_c ( ikkwd, 0, secnum[0]*3,  &n, secdir, &found );
   if ( ! found ) {
     setmsg_c ( "Loaded IK does not contain #." );
     errch_c  ( "#", ikkwd );
     sigerr_c ( "NOIKDATA" );
   }

} /* End of "get IK data" */
