/*
  This small example computes direction to the Sun in the s/c frame 
  and angle between the Sun and view directions of the sectors of 
  specified ASPERA sensor.

  Boris Semenov, NAIF/JPL, July 20, 2003.

*/

#include <stdio.h>
#include <string.h>
#include "SpiceUsr.h"

#define   LINESZ  257
#define   WORDSZ  33
#define   MAXLOK  32

/*
  Auxiliary routines to make example less cluttered.
*/
void getik( ConstSpiceChar    * sensor,
            SpiceChar         * sensnm,
            SpiceInt          * sensid,
            SpiceInt            secnum[2],
            SpiceDouble         secsiz[2],
            SpiceChar         * secfrm,
            SpiceDouble       * secdir );

void checkck( SpiceDouble         et,
            SpiceBoolean      * gotscatt, 
            SpiceBoolean      * gotscanatt );

/*
  Main.
*/
int main( int argc, char * argv[] )
{

  /*
    Command line inputs.
  */
  SpiceChar               mkfile [LINESZ];
  SpiceChar               sensor [WORDSZ];
  SpiceChar               utc    [WORDSZ];
  SpiceDouble             angle;

  /*
    Sensor data.
  */
  SpiceChar               sensnm [WORDSZ];
  SpiceInt                sensid;
  SpiceInt                secnum [2];
  SpiceDouble             secsiz [2];
  SpiceChar               secfrm [WORDSZ];
  SpiceDouble             secdir [MAXLOK][3];

  /*
    Orientation availability flags.
  */
  SpiceBoolean            gotscatt;
  SpiceBoolean            gotscanatt;
  SpiceBoolean            gotangle;

  /*
    Other things.
  */
  SpiceDouble             et;
  SpiceDouble             sundir [3];
  SpiceDouble             lt;
  SpiceInt                i;
  SpiceDouble             a2smat [3][3];
  SpiceDouble             m1 [3][3];
  SpiceDouble             m2 [3][3];
  SpiceDouble             m3 [3][3];
  SpiceDouble             hvec [3];

  /*
    If command line doesn't contain 3 or 4 arguments, display usage.
  */
  if      ( argc < 4 || argc > 5 )
  {
    printf ( "                                                      \n" );
    printf ( "Usage:                                                \n" );
    printf ( "                                                      \n" );
    printf ( "   > spice_example <furnsh> <sensor> <utc> [<angle>]  \n" );
    printf ( "                                                      \n" );
    printf ( "where:                                                \n" );
    printf ( "      furnsh -- name of the furnsh file               \n" );
    printf ( "      sensor -- ELS, NPI, NPD, or IMA                 \n" );
    printf ( "      utc    -- time as one word (YYYY-MM-DDTHR:MN:SC)\n" );
    printf ( "      angle  -- scanner angle in degrees (optional)   \n" );
    printf ( "                                                      \n" );
    return(1);
  } 

  /*
    Save required command line arguments -- furnsh file name, sensor 
    acronym, and input UTC time.
  */
  strcpy( mkfile, argv[1] );
  strcpy( sensor, argv[2] );
  strcpy( utc,    argv[3] );

  /*
    If the scanner angle have been provided, save it too and set a 
    flag indicating that we got it.
  */ 
  if ( argc == 5 ) {
    gotangle = 1;
    prsdp_c( argv[4], &angle );
  } else {
    gotangle = 0;
  }  

  /*
    Load furnsh file listing SPICE kernels.
  */
  furnsh_c ( mkfile );

  /*
    Call getik function for retrieve IK data for specified sensor.
  */
  getik( sensor, sensnm, &sensid, secnum, secsiz, secfrm, 
         ( SpiceDouble * ) secdir );

  /*
    Report all buffered sensor data.
  */
  printf( "SENSOR NAIF NAME:  %s\n", sensnm );
  printf( "SENSOR NAIF ID:    %d\n", (int ) sensid );
  printf( "NUMBER OF SECTORS: %d %d\n", (int ) secnum[0], (int ) secnum[1] );
  printf( "SECTOR SIZE:       %12.3f %12.3f\n", secsiz[0], secsiz[1] );
  printf( "REFERENCE FRAME:   %s\n", secfrm );
  printf( "SECTOR VIEW DIRECTIONS:\n" );
  for ( i = 0; i < secnum[0]; i++ )
  {
    printf( "   %12.9f %12.9f %12.9f\n", 
            secdir[i][0], secdir[i][1], secdir[i][2] );
  }

  /*
    Convert input UTC to ET.
  */
  str2et_c ( utc, &et );

  /*
    Check if MEX s/c and ASPERA scanner CK data is available.
  */
  checkck( et, &gotscatt, &gotscanatt );

  /*
    Report CK data availability.
  */
  printf ( "SPACECRAFT ATTITUDE: %d\n", gotscatt );
  printf ( "SCANNER ATTITUDE:    %d\n", gotscanatt );
  printf ( "SCANNER ANGLE:       %d\n", gotangle );

  /*
    Check if we have s/c orientation. If not, we cannot compute
    anything.
  */
  if ( ! gotscatt ) 
  {
    printf ( "S/c orientation is not available at specified time.\n" );
    return(1);
  }

  /*
    Check if we have scanner orientation or scanner angle. If we don't
    have one or the other, we cannot compute anything.
  */
  if ( ! ( gotscanatt || gotangle ) )
  {
    printf ( "Scanner orientation is not available at specified time.\n" );
    return(1);
  }

  /*
    Compute direction from s/c to the Sun in s/c frame with a single
    call to to spkpos_c.
  */
  spkpos_c ( "SUN", et, "MEX_SPACECRAFT", "LT+S", "MEX", sundir, &lt );
  vhat_c   ( sundir, sundir );

  printf ( "SUN IN S/C FRAME:   %12.8f %12.8f %12.8f\n", 
           sundir[0], sundir[1], sundir[2] );

  /*
    Compute sensor frame orientation w.r.t. to the s/c frame. If we
    have scanner CK data or if our sensor is IMA, it takes just one
    call. If we have scanner angle and sensor is not IMA, then it's a
    bit longer.
  */  
  if ( gotscanatt || eqstr_c( sensor, "IMA" ) )
  {
    /*
      One call to pxform returning matrix rotating vectors from sensor 
      frame to the s/c frame does the job.
    */
    pxform_c ( secfrm, "MEX_SPACECRAFT", et, a2smat );
  }
  else 
  {
    /*
      To "get" from scanner to the s/c frame we need to combine three
      orientations -- sensor wrt to scanner frame, scanner wrt to the
      base, and base wrt to the s/c. The first and the third can be
      computed using pxform_c. The middle one is a single rotation by
      negative of "specified angle + 180 degrees" about Z.
    */
    pxform_c ( secfrm,            "MEX_ASPERA_SCANNER", et, m1 );
    eul2m_c  ( 0.0, 0.0, -(angle+180.0)/dpr_c(), 1, 2, 3,     m2 );
    pxform_c ( "MEX_ASPERA_BASE", "MEX_SPACECRAFT",     et, m3 );
   
    mxm_c    ( m2, m1,     a2smat );
    mxm_c    ( m3, a2smat, a2smat );
    
  }
 
  /*
    Now with rotation from the sensor to the s/c frame in hand each of
    sector view directions can be transformed to the s/c frame to find
    the angle between it and the Sun direction.
  */
  printf ( "SECTOR DIRECTIONS IN S/C FRAME AND SUN-SECTOR DIR ANGLE:\n" ); 
  for ( i = 0; i < secnum[0]; i++ )
  {
    mxv_c ( a2smat, secdir[i], hvec );
    printf( "   %12.9f %12.9f %12.9f %12.3f\n", 
            hvec[0], hvec[1], hvec[2], vsep_c( hvec, sundir )*dpr_c() );
  }

  return ( 0 );

}
